package cn.iflyos.open.library.quiet

import android.content.Context
import android.media.*
import android.os.Build
import android.util.Log
import cn.iflyos.open.library.EncoderConfig
import cn.iflyos.open.library.QuietEncoder
import okio.Okio
import okio.Pipe

class SoundWavePlayer(private val context: Context) {

    private val track: AudioTrack

    private var pipe: Pipe? = null

    private var cacheByteArray = ByteArray(0)

    private var playThread: Thread? = null
    private var prepareThread: Thread? = null

    var isPlaying = false
        private set

    companion object {
        private const val TAG = "SoundWavePlayer"

        private const val MAX_BUFFER_SIZE = 256L * 1024
        private const val CHANNEL = AudioFormat.CHANNEL_OUT_MONO
        private const val SAMPLE_RATE = 16000
        private const val ENCODING = AudioFormat.ENCODING_PCM_16BIT
        private val MIN_BUFFER_SIZE = AudioTrack.getMinBufferSize(SAMPLE_RATE, CHANNEL, ENCODING)
    }

    var onPlayStateChangeListener: OnPlayStateChangeListener? = null

    private val focusListener = AudioManager.OnAudioFocusChangeListener { focusChange ->
        when (focusChange) {
            AudioManager.AUDIOFOCUS_LOSS_TRANSIENT -> {
            }
            AudioManager.AUDIOFOCUS_GAIN -> {
            }
            AudioManager.AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK -> {
            }
            AudioManager.AUDIOFOCUS_LOSS -> {
                //Stop playback
                stop()
            }
        }
    }

    private val audioFocusRequest: AudioFocusRequest? =
            if (Build.VERSION.SDK_INT >= 26)
                AudioFocusRequest.Builder(AudioManager.AUDIOFOCUS_GAIN)
                        .setAudioAttributes(AudioAttributes.Builder()
                                .setLegacyStreamType(AudioManager.STREAM_MUSIC)
                                .build())
                        .setOnAudioFocusChangeListener(focusListener)
                        .build()
            else null

    init {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
            @Suppress("DEPRECATION")
            this.track = AudioTrack(
                    AudioManager.STREAM_MUSIC, SAMPLE_RATE, CHANNEL, ENCODING,
                    MIN_BUFFER_SIZE,
                    AudioTrack.MODE_STREAM)
        } else {
            this.track = AudioTrack.Builder()
                    .setAudioAttributes(AudioAttributes.Builder()
                            .setContentType(AudioAttributes.CONTENT_TYPE_MUSIC)
                            .build())
                    .setAudioFormat(AudioFormat.Builder()
                            .setSampleRate(SAMPLE_RATE)
                            .setEncoding(ENCODING)
                            .setChannelMask(AudioFormat.CHANNEL_OUT_MONO)
                            .build())
                    .setBufferSizeInBytes(MIN_BUFFER_SIZE)
                    .setTransferMode(AudioTrack.MODE_STREAM)
                    .build()
        }
    }

    interface PrepareListener {
        fun onPrepared(player: SoundWavePlayer)
    }

    fun prepare(reAuth: Boolean, ssid: String, password: String?, listener: PrepareListener) {
        prepare("${if (reAuth) "1" else "0"}\n$ssid\n${if (password.isNullOrEmpty()) "" else password}", listener)
    }

    private fun prepare(data: String, listener: PrepareListener) {
        stop()

        try {
            prepareThread?.interrupt()
        } catch (e: Exception) {
            e.printStackTrace()
        }

        val encoder = QuietEncoder(EncoderConfig(context, "audible"), SAMPLE_RATE)

        prepareThread = object : Thread() {
            override fun run() {
                try {
                    val pipe = Pipe(MAX_BUFFER_SIZE)

                    this@SoundWavePlayer.pipe = pipe

                    encoder.send(data.toByteArray())

                    val sink = Okio.buffer(pipe.sink())

                    val frame = ByteArray(32768)
                    var emitted: Int = encoder.emit(frame).toInt()
                    while (emitted > 0) {
                        sink.write(frame, 0, emitted)

                        emitted = encoder.emit(frame).toInt()
                    }
                    sink.close()

                    encoder.destroy()
                } catch (e: Exception) {
                    e.printStackTrace()
                }

                listener.onPrepared(this@SoundWavePlayer)

                if (prepareThread == this) {
                    prepareThread = null
                }
            }
        }
        prepareThread?.start()
    }

    fun play() {
        if (isPlaying)
            stop()
        when {
            pipe == null && cacheByteArray.isEmpty() ->
                throw IllegalStateException("Player not prepared!")
            requestFocus() != AudioManager.AUDIOFOCUS_REQUEST_GRANTED ->
                Log.w(TAG, "Request audio focus failed")
            else -> {
                Log.d(TAG, "SoundWavePlayer start playing")

                // start playing
                track.play()
                onPlayStateChangeListener?.onPlayStateChanged(AudioTrack.PLAYSTATE_PLAYING)

                isPlaying = true

                playThread = object : Thread() {
                    override fun run() {
                        try {
                            if (track.state != AudioTrack.STATE_INITIALIZED) {
                                Log.e(TAG, "AudioTrack has not initialized!")
                                return
                            }
                            val pipe = this@SoundWavePlayer.pipe
                            if (pipe != null) {
                                val source = Okio.buffer(pipe.source())
                                val byteArray = source.readByteArray()

                                this@SoundWavePlayer.pipe = null

                                if (!byteArray.isEmpty()) {
                                    cacheByteArray = byteArray
                                }
                            }
                            if (!cacheByteArray.isEmpty()) {
                                val dividerArray = arrayOf(300L, 150, 150, 500)
                                var i = 0
                                while (isPlaying && track.playState == AudioTrack.PLAYSTATE_PLAYING) {
                                    if (track.state != AudioTrack.STATE_INITIALIZED) {
                                        Log.e(TAG, "AudioTrack has not initialized!")
                                        return
                                    }
                                    track.write(cacheByteArray, 0, cacheByteArray.size)
                                    Thread.sleep(dividerArray[i % dividerArray.size])
                                    i++
                                }
                            }
                            if (playThread == this) {
                                playThread = null
                            }
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }
                }
                playThread?.start()
            }
        }
    }

    fun stop() {
        try {
            playThread?.interrupt()
        } catch (e: Exception) {
            e.printStackTrace()
        }

        if (abandonFocus() != AudioManager.AUDIOFOCUS_REQUEST_GRANTED) {
            Log.w(TAG, "Abandon audio focus failed")
        }

        Log.d(TAG, "SoundWavePlayer stop playing")

        onPlayStateChangeListener?.onPlayStateChanged(AudioTrack.PLAYSTATE_STOPPED)

        isPlaying = false

        if (track.state != AudioTrack.STATE_UNINITIALIZED) {
            track.stop()
            track.flush()
        }
    }

    fun destroy() {
        isPlaying = false

        track.release()
    }

    interface OnPlayStateChangeListener {
        fun onPlayStateChanged(state: Int)
    }

    private fun requestFocus(): Int {
        val audioManager = context.applicationContext.getSystemService(Context.AUDIO_SERVICE) as? AudioManager
        if (audioManager == null) {
            Log.w(TAG, "Request audio focus failed: cannot get AudioManager")
            return AudioManager.AUDIOFOCUS_REQUEST_FAILED
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val audioFocusRequest = this.audioFocusRequest
                    ?: return AudioManager.AUDIOFOCUS_REQUEST_FAILED
            val result = audioManager.requestAudioFocus(audioFocusRequest)
            Log.w(TAG, "Request audio focus result: $result")
            return result
        } else {
            val result = audioManager.requestAudioFocus(focusListener,
                    AudioManager.STREAM_MUSIC, AudioManager.AUDIOFOCUS_GAIN)
            Log.w(TAG, "Request audio focus result: $result")
            return result
        }
    }

    private fun abandonFocus(): Int {
        val audioManager = context.getSystemService(Context.AUDIO_SERVICE) as? AudioManager
                ?: return AudioManager.AUDIOFOCUS_REQUEST_FAILED
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val audioFocusRequest = this.audioFocusRequest
                    ?: return AudioManager.AUDIOFOCUS_REQUEST_FAILED
            return audioManager.abandonAudioFocusRequest(audioFocusRequest)
        } else {
            return audioManager.abandonAudioFocus(focusListener)
        }
    }
}