package com.iflytek.cyber.home.sdk.demo

import android.Manifest
import android.app.Service
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.location.LocationManager
import android.os.Binder
import android.os.IBinder
import android.os.ParcelUuid
import android.util.Log
import androidx.core.content.PermissionChecker
import no.nordicsemi.android.ble.callback.DataReceivedCallback
import no.nordicsemi.android.ble.callback.FailCallback
import no.nordicsemi.android.ble.error.GattError
import no.nordicsemi.android.support.v18.scanner.*

class BleService : Service() {
    @Suppress("unused") // some error reason wouldn't be used
    companion object {
        private const val TAG = "BleService"

        private const val ACTION_PREFIX = "com.iflytek.home.app.device.config.net.ble2.action"
        const val ACTION_START = "$ACTION_PREFIX.START"
        const val ACTION_CONNECT = "$ACTION_PREFIX.CONNECT"
        const val ACTION_DISCONNECT = "$ACTION_PREFIX.DISCONNECT"
        const val ACTION_SETUP = "$ACTION_PREFIX.SETUP"
        const val ACTION_REQUEST_DEVICE_ID = "$ACTION_PREFIX.REQUEST_DEVICE_ID"
        const val ACTION_STOP = "$ACTION_PREFIX.STOP"

        // broadcast
        const val ACTION_DEVICE_ID_RECEIVED = "$ACTION_PREFIX.DEVICE_ID_FOUND"
        const val ACTION_REQUEST_DEVICE_ID_FAILED = "$ACTION_PREFIX.REQUEST_DEVICE_ID_FAILED"
        const val ACTION_DEVICE_FOUND = "$ACTION_PREFIX.DEVICE_FOUND"
        const val ACTION_DEVICE_CONNECTED = "$ACTION_PREFIX.DEVICE_CONNECTED"
        const val ACTION_DEVICE_CONNECT_FAILED = "$ACTION_PREFIX.DEVICE_CONNECT_FAILED"
        const val ACTION_DEVICE_DISCONNECTED = "$ACTION_PREFIX.DEVICE_DISCONNECTED"
        const val ACTION_SETUP_FAILED = "$ACTION_PREFIX.SETUP_FAILED"

        const val EXTRA_UUID = "extra_uuid"
        const val EXTRA_SSID = "extra_ssid"
        const val EXTRA_PASSWORD = "extra_password"
        const val EXTRA_DEVICE_CODE = "extra_device_code"
        const val EXTRA_DEVICE = "extra_device"
        const val EXTRA_ERROR = "extra_error"
        const val EXTRA_REASON = "extra_reason"
        const val EXTRA_DEVICE_ID = "extra_device_id"

        const val REASON_DEVICE_DISCONNECTED = -1
        const val REASON_DEVICE_NOT_SUPPORTED = -2
        const val REASON_NULL_ATTRIBUTE = -3
        const val REASON_REQUEST_FAILED = -4
        const val REASON_TIMEOUT = -5
        const val REASON_VALIDATION = -6
        const val REASON_BLUETOOTH_DISABLED = -100
    }

    var isScanning = false
        private set
    private var manager: LeSetupManager2? = null

    private val binder = BleServiceBinder(this)
    private var needDeviceId = false
    private var serviceDiscovered = false

    private var scanCallback = generateScanCallback()

    class BleServiceBinder(val service: BleService) : Binder()

    override fun onBind(intent: Intent): IBinder? {
        return binder
    }

    override fun onCreate() {
        super.onCreate()

        val intentFilter = IntentFilter()
        intentFilter.addAction(BluetoothAdapter.ACTION_STATE_CHANGED)
        registerReceiver(mReceiver, intentFilter)
    }

    fun isConnected(): Boolean? {
        return manager?.isConnected
    }

    /**
     * Location service if enable
     *
     * @param context context
     * @return location is enable if return true, otherwise disable.
     */
    private fun isLocationEnable(context: Context): Boolean {
        val locationManager = context.getSystemService(Context.LOCATION_SERVICE) as LocationManager
        val networkProvider = locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)
        val gpsProvider = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)
        return networkProvider || gpsProvider
    }

    private fun hasReady(): Boolean {
        if (!BluetoothAdapter.getDefaultAdapter().isEnabled) {
            Log.e(TAG, "Bluetooth is not enabled.")
            return false
        }
        if (!isLocationEnable(this)) {
            Log.e(TAG, "Location Service is not enabled.")
            return false
        }
        if (PermissionChecker.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                != PermissionChecker.PERMISSION_GRANTED) {

            Log.e(TAG, "Location permission is not granted.")
            return false
        }
        return true
    }

    fun startScan() {
        if (hasReady()) {
            if (isScanning) {
                Log.e(TAG, "Ble scanner already started")
                return
            }
            isScanning = true

            BluetoothLeScannerCompat.getScanner().startScan(
                    listOf(
                            ScanFilter.Builder()
                                    .setServiceUuid(ParcelUuid(LeSetupManager2.IFLYOS_SETUP_SERVICE))
                                    .build()
                    ),
                    ScanSettings.Builder()
                            .setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY)
                            .build(),
                    scanCallback
            )
        }
    }

    fun stopScan() {
        if (hasReady()) {
            if (!isScanning) {
                Log.e(TAG, "Ble scanner not started")
                return
            }
            isScanning = false

            BluetoothLeScannerCompat.getScanner().stopScan(scanCallback)
        }
    }

    // after disable bluetooth, callback should be re-generate
    private fun generateScanCallback() = object : ScanCallback() {

        override fun onScanFailed(errorCode: Int) {
            super.onScanFailed(errorCode)
            Log.e("Ble", "onScanFailed: $errorCode")

            BleDeviceStorage.clear()
        }

        override fun onScanResult(callbackType: Int, result: ScanResult) {
            super.onScanResult(callbackType, result)
            val data = result.scanRecord?.manufacturerSpecificData?.get(0xAAAA)
            data?.let {
                if (it.size == 16) {
                    val intent = Intent()
                    intent.action = ACTION_DEVICE_FOUND

                    val clientId = formatUUID(bytes2HexString(it))

                    BleDeviceStorage.addDevice(result.device, clientId)

                    intent.putExtra(EXTRA_DEVICE, result.device)
                    intent.putExtra(EXTRA_UUID, clientId)
                    sendBroadcast(intent)
                }
            }
        }
    }

    private val bleListener = object : LeSetupManager2.Callback() {
        override fun onDeviceConnected(device: BluetoothDevice) {
            super.onDeviceConnected(device)

            serviceDiscovered = false

            val broadcast = Intent(ACTION_DEVICE_CONNECTED)
            broadcast.putExtra(EXTRA_DEVICE, device)
            sendBroadcast(broadcast)
        }

        override fun onDeviceDisconnected(device: BluetoothDevice) {
            super.onDeviceDisconnected(device)

            serviceDiscovered = false

            val broadcast = Intent(ACTION_DEVICE_DISCONNECTED)
            broadcast.putExtra(EXTRA_DEVICE, device)
            sendBroadcast(broadcast)
        }

        override fun onServicesDiscovered(device: BluetoothDevice, optionalServicesFound: Boolean) {
            serviceDiscovered = true

            if (needDeviceId) {
                requestDeviceId()
            }

            super.onServicesDiscovered(device, optionalServicesFound)
        }

        override fun onDeviceConnectFailed(device: BluetoothDevice, status: Int) {
            super.onDeviceConnectFailed(device, status)

            val broadcast = Intent(ACTION_DEVICE_CONNECT_FAILED)
            broadcast.putExtra(EXTRA_REASON, status)
            broadcast.putExtra(EXTRA_DEVICE, device)
            sendBroadcast(broadcast)
        }
    }

    @Suppress("SpellCheckingInspection")
    fun bytes2HexString(b: ByteArray): String {
        val hex = "0123456789abcdef".toByteArray()
        val buff = ByteArray(2 * b.size)
        for (i in b.indices) {
            buff[2 * i] = hex[b[i].toInt() shr 4 and 0x0f]
            buff[2 * i + 1] = hex[b[i].toInt() and 0x0f]
        }
        return String(buff)
    }

    fun formatUUID(hexString: String): String {
        return "${hexString.substring(0, 8)}-${hexString.substring(8, 12)}-${hexString.substring(
                12,
                16
        )}-${hexString.substring(16, 20)}-${hexString.substring(20, 32)}"
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        when (intent?.action) {
            ACTION_START -> {
                startScan()
            }
            ACTION_STOP -> {
                stopScan()
            }
            ACTION_CONNECT -> {
                intent.getParcelableExtra<BluetoothDevice>(EXTRA_DEVICE)?.let { device ->
                    if (manager != null) {
                        manager?.disconnect()?.enqueue()
                        manager?.destroy()
                    }
                    manager = LeSetupManager2.connectDevice(this, device, bleListener)
                }
            }
            ACTION_DISCONNECT -> {
                BleDeviceStorage.clear()

                manager?.sendDisconnect()
            }
            ACTION_SETUP -> {
                val ssid = intent.getStringExtra(EXTRA_SSID)
                val password = intent.getStringExtra(EXTRA_PASSWORD)
                val deviceCode = intent.getStringExtra(EXTRA_DEVICE_CODE)
                manager?.setupData(ssid, password, deviceCode)
            }
            ACTION_REQUEST_DEVICE_ID -> {
                if (serviceDiscovered) {
                    requestDeviceId()
                } else {
                    needDeviceId = true
                }
            }
        }
        return super.onStartCommand(intent, flags, startId)
    }

    private fun requestDeviceId() {
        manager?.readDeviceIdSync(
                DataReceivedCallback { _, data ->
                    val value = data.value
                            ?: return@DataReceivedCallback
                    val broadcast = Intent(ACTION_DEVICE_ID_RECEIVED)
                    broadcast.putExtra(EXTRA_DEVICE_ID, String(value))
                    sendBroadcast(broadcast)
                },
                FailCallback { _, status ->
                    Log.e(TAG, "Read device id failed, status: $status")
                    when (status) {
                        REASON_DEVICE_NOT_SUPPORTED, GattError.GATT_CONN_L2C_FAILURE -> {
                            manager?.refreshBleDeviceCache()
                                    ?.done {
                                        manager?.readDeviceIdSync(
                                                DataReceivedCallback { _, data ->
                                                    val value = data.value
                                                            ?: return@DataReceivedCallback
                                                    val broadcast = Intent(ACTION_DEVICE_ID_RECEIVED)
                                                    broadcast.putExtra(EXTRA_DEVICE_ID, String(value))
                                                    sendBroadcast(broadcast)
                                                })
                                    }
                                    ?.enqueue()
                        }
                        else -> {
                            val broadcast = Intent(ACTION_REQUEST_DEVICE_ID_FAILED)
                            broadcast.putExtra(EXTRA_REASON, status)
                            sendBroadcast(broadcast)
                        }
                    }
                })
    }

    override fun onDestroy() {
        super.onDestroy()

        manager?.destroy()

        unregisterReceiver(mReceiver)
    }

    private val mReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            when (intent?.action) {
                BluetoothAdapter.ACTION_STATE_CHANGED -> {
                    val state = intent.getIntExtra(BluetoothAdapter.EXTRA_STATE,
                            BluetoothAdapter.ERROR)
                    when (state) {
                        BluetoothAdapter.STATE_OFF -> {
                            BleDeviceStorage.clear()

                            isScanning = false

                            scanCallback = generateScanCallback()
                        }
                    }
                }
            }
        }

    }
}