package com.iflytek.cyber.home.sdk.demo

import android.annotation.SuppressLint
import android.bluetooth.BluetoothDevice
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.bumptech.glide.Glide
import com.google.gson.JsonParser
import com.iflytek.home.sdk.IFlyHome
import com.iflytek.home.sdk.callback.ResponseCallback
import kotlinx.android.synthetic.main.activity_handle_ble_device.*
import no.nordicsemi.android.ble.error.GattError
import org.json.JSONObject
import retrofit2.Call
import retrofit2.Response

class HandleBleDeviceActivity : AppCompatActivity() {
    private var device: BluetoothDevice? = null
    private var clientId: String? = null
    private var deviceId: String? = null
    private var deviceCode: String? = null

    private var isShown = false

    companion object {
        private const val TAG = "HandleBleDeviceActivity"
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (Build.VERSION.SDK_INT >= 23)
            window.decorView.systemUiVisibility = window.decorView.systemUiVisibility or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
        setContentView(R.layout.activity_handle_ble_device)

        supportActionBar?.run {
            setHomeButtonEnabled(true)
            setDisplayHomeAsUpEnabled(true)
        }

        device = intent.getParcelableExtra("device")
        clientId = intent.getStringExtra("client_id")

        clientId?.let { clientId ->
            BleDeviceStorage.getClientInfo(clientId) { clientInfo ->
                @SuppressLint("SetTextI18n")
                device_info.text = """
                    设备信息
                    设备名：${clientInfo.clientName}
                    client id: ${clientInfo.clientId}
                """.trimIndent()

                Glide.with(this)
                        .load(clientInfo.clientImage)
                        .into(device_image)
            }
        } ?: run {
            toast("client id 为空", Toast.LENGTH_SHORT)
            finish()
        }

        device?.let {
            val connect = Intent(this, BleService::class.java)
            connect.action = BleService.ACTION_CONNECT
            connect.putExtra(BleService.EXTRA_DEVICE, it)
            startService(connect)

            device_status.setText(R.string.connecting)
        } ?: run {
            toast("蓝牙设备为空", Toast.LENGTH_SHORT)
            finish()
        }

        send.setOnClickListener {
            val clientId = this.clientId
            val deviceId = this.deviceId
            when {
                clientId.isNullOrEmpty() -> {
                    // 不应为空
                    toast("设备信息不完善，请稍候重试")
                }
                deviceId.isNullOrEmpty() -> {
                    val requestDeviceId = Intent(this, BleService::class.java)
                    requestDeviceId.action = BleService.ACTION_REQUEST_DEVICE_ID
                    startService(requestDeviceId)
                }
                else -> {
                    // 如果输入过程过长，可能会导致授权码发送到设备，设备联网成功后会过期
                    // 所以点击发送之后应该重新获取一次授权码
                    IFlyHome.getDeviceCode(clientId, deviceId, object : ResponseCallback {
                        override fun onResponse(response: Response<String>) {
                            if (response.isSuccessful) {
                                val json = JsonParser().parse(response.body()).asJsonObject

                                deviceCode = json.get("device_code").asString
                                device_code.text = deviceCode

                                val setup = Intent(this@HandleBleDeviceActivity, BleService::class.java)
                                setup.action = BleService.ACTION_SETUP
                                setup.putExtra(BleService.EXTRA_DEVICE_CODE, deviceCode)
                                setup.putExtra(BleService.EXTRA_SSID, wifi_name.text.toString())
                                setup.putExtra(BleService.EXTRA_PASSWORD, wifi_password.text.toString())
                                startService(setup)
                            } else {
                                response.errorBody()?.string()?.let { errorBody ->
                                    val json = JSONObject(errorBody)
                                    toast("${json.optString("title")}:${json.optString("message")}")
                                }

                                val connect = Intent(this@HandleBleDeviceActivity, BleService::class.java)
                                connect.action = BleService.ACTION_DISCONNECT
                                startService(connect)

                                toast("添加失败\n设备连接时出现错误")
                            }
                        }

                        override fun onFailure(call: Call<String>, t: Throwable) {
                            toast("无法验证设备，请稍候重试")
                        }
                    })
                }
            }
        }

        get_device_code.setOnClickListener {
            val clientId = clientId ?: run {
                toast("client id 为空")
                return@setOnClickListener
            }
            val deviceId = deviceId ?: run {
                toast("device id 为空")
                return@setOnClickListener
            }
            getDeviceCode(clientId, deviceId)
        }

        get_device_id.setOnClickListener {
            val requestDeviceId = Intent(this, BleService::class.java)
            requestDeviceId.action = BleService.ACTION_REQUEST_DEVICE_ID
            startService(requestDeviceId)
        }

        check_device_code.setOnClickListener{
            val deviceCode = this.deviceCode
            if (deviceCode.isNullOrEmpty()) {
                toast("请选获取设备授权码")
            } else {
                IFlyHome.getDeviceConfirm(deviceCode, object : ResponseCallback{
                    override fun onResponse(response: Response<String>) {
                        if (response.isSuccessful) {
                            device_code_state.text = response.body()
                        } else {
                            val errorBody = response.errorBody()?.string()
                            if (errorBody.isNullOrEmpty()) {
                                device_code_state.text = "请求出错"
                            } else {
                                device_code_state.text = errorBody
                            }
                        }
                    }

                    override fun onFailure(call: Call<String>, t: Throwable) {
                        device_code_state.text = t.message
                    }
                })
            }
        }

        val intentFilter = IntentFilter()
        intentFilter.addAction(BleService.ACTION_DEVICE_CONNECTED)
        intentFilter.addAction(BleService.ACTION_DEVICE_CONNECT_FAILED)
        intentFilter.addAction(BleService.ACTION_DEVICE_ID_RECEIVED)
        intentFilter.addAction(BleService.ACTION_REQUEST_DEVICE_ID_FAILED)
        intentFilter.addAction(BleService.ACTION_DEVICE_DISCONNECTED)
        registerReceiver(bleReceiver, intentFilter)
    }

    override fun onResume() {
        super.onResume()

        isShown = true
    }

    override fun onPause() {
        super.onPause()

        isShown = false
    }

    override fun onDestroy() {
        super.onDestroy()

        unregisterReceiver(bleReceiver)

        val connect = Intent(this, BleService::class.java)
        connect.action = BleService.ACTION_DISCONNECT
        startService(connect)
    }

    private fun getDeviceCode(clientId: String, deviceId: String) {
        IFlyHome.getDeviceCode(clientId, deviceId, object : ResponseCallback {
            override fun onResponse(response: Response<String>) {
                if (response.isSuccessful) {
                    val json = JsonParser().parse(response.body()).asJsonObject
                    deviceCode = json.get("device_code").asString

                    device_code.text = deviceCode

                    send.isEnabled = true
                } else {
                    response.errorBody()?.string()?.let { errorBody ->
                        try {
                            val json = JSONObject(errorBody)
                            toast("${json.optString("title")}:${json.optString("message")}")
                        } catch (e: Exception) {
                            toast("请求设备授权码失败，未知错误")
                        }
                    }

                    val connect = Intent(this@HandleBleDeviceActivity, BleService::class.java)
                    connect.action = BleService.ACTION_DISCONNECT
                    startService(connect)

                    toast("添加失败\n设备连接时出现错误")
                }
            }

            override fun onFailure(call: Call<String>, t: Throwable) {
                BleDeviceStorage.clear()

                val connect = Intent(this@HandleBleDeviceActivity, BleService::class.java)
                connect.action = BleService.ACTION_DISCONNECT
                startService(connect)

                toast("添加失败\n设备连接时出现错误")
            }
        })
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        if (item?.itemId == android.R.id.home)
            onBackPressed()
        return super.onOptionsItemSelected(item)
    }

    private val bleReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {

            when (intent.action) {
                BleService.ACTION_DEVICE_CONNECTED -> {
                    val requestDeviceId = Intent(context, BleService::class.java)
                    requestDeviceId.action = BleService.ACTION_REQUEST_DEVICE_ID
                    context.startService(requestDeviceId)

                    device_status.setText(R.string.connected)
                }
                BleService.ACTION_DEVICE_CONNECT_FAILED -> {
                    BleDeviceStorage.clear()

                    when (intent.getIntExtra(BleService.EXTRA_REASON, -1)) {
                        BleService.REASON_TIMEOUT, GattError.GATT_ERROR -> {
                            context.toast("添加失败\n设备连接时出现错误")
                        }
                        GattError.GATT_VALUE_OUT_OF_RANGE -> {
                            context.toast("添加失败\n请靠近设备后尝试连接")
                        }
                        else -> {
                            context.toast("添加失败\n设备连接时出现错误")
                        }
                    }

                    send.isEnabled = false
                }
                BleService.ACTION_REQUEST_DEVICE_ID_FAILED -> {
                    BleDeviceStorage.clear()

                    val reason = intent.getIntExtra(BleService.EXTRA_REASON, -1)
                    Log.w(TAG, "Request device id failed, status: $reason")

                    val connect = Intent(context, BleService::class.java)
                    connect.action = BleService.ACTION_DISCONNECT
                    context.startService(connect)

                    context.toast("添加失败\n无法从设备获取设备 id")

                    send.isEnabled = false
                }
                BleService.ACTION_DEVICE_DISCONNECTED -> {
                    device_status.setText(R.string.disconnected)

                    send.isEnabled = false
                }
                BleService.ACTION_DEVICE_ID_RECEIVED -> {
                    val deviceId = intent.getStringExtra(BleService.EXTRA_DEVICE_ID)

                    if (deviceId.isNullOrEmpty()) {
                        context.toast("请求设备 id 为空，请重试")

                        val connect = Intent(context, BleService::class.java)
                        connect.action = BleService.ACTION_DISCONNECT
                        context.startService(connect)
                    } else {
                        this@HandleBleDeviceActivity.deviceId = deviceId
                        device_id.text = deviceId

                        // 获取到设备 id 后，应当尝试获取一次设备授权码
                        // * 如果成功，可以肯定各部分数据都是正确的，流程可以继续往下走
                        // * 如果失败，说明 client id 可能有部分配置是错误的，在错误信息中会返回相关信息
                        getDeviceCode(clientId!!, deviceId)
                    }
                }
            }
        }
    }
}