package com.iflytek.cyber.home.sdk.demo

import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothGatt
import android.bluetooth.BluetoothGattCharacteristic
import android.content.Context
import android.os.Handler
import android.os.HandlerThread
import android.util.Log
import no.nordicsemi.android.ble.BleManager
import no.nordicsemi.android.ble.BleManagerCallbacks
import no.nordicsemi.android.ble.Request
import no.nordicsemi.android.ble.callback.DataReceivedCallback
import no.nordicsemi.android.ble.callback.DataSentCallback
import no.nordicsemi.android.ble.callback.FailCallback
import no.nordicsemi.android.ble.data.Data
import no.nordicsemi.android.ble.error.GattError
import java.util.*

class LeSetupManager2 private constructor(
        context: Context,
        @Suppress("MemberVisibilityCanBePrivate") val device: BluetoothDevice,
        listener: Callback? = null
) : BleManager<BleManagerCallbacks>(context) {

    companion object {
        private const val TAG = "LeSetupManager2"

        internal val IFLYOS_SETUP_SERVICE = UUID.fromString("00001ff9-0000-1000-8000-00805f9b34fb")
        internal val IFLYOS_SETUP_REQUEST = UUID.fromString("00001ffa-0000-1000-8000-00805f9b34fb")

        private const val KEEP_ALIVE = "keep-alive"

        fun connectDevice(
                context: Context, device: BluetoothDevice,
                listener: Callback? = null
        ): LeSetupManager2 {
            return LeSetupManager2(context, device, listener)
        }
    }

    private var listener: BleManagerCallbacks? = null
    private var isDestroyed = false
    private var handler: Handler? = null
    private val keepAliveRunnable = object : Runnable {
        override fun run() {
            if (isConnected) {
                if (characteristic != null) {
                    Data.opCode(2)
                    writeCharacteristic(characteristic, Data.from(KEEP_ALIVE))
                            .with { _, _ -> }
                            .done { }
                            .fail { _, status ->
                                Log.w(TAG, "SEND $KEEP_ALIVE failed, status: $status, ${GattError.parse(status)}")
                            }
                            .enqueue()
                }
            }
            if (!isDestroyed)
                handler?.postDelayed(this, 5000)
        }
    }
    private var characteristic: BluetoothGattCharacteristic? = null
    private val gattCallback = object : BleManagerGattCallback() {

        override fun initialize() {
            requestMtu(517)
                    .with { _, mtu ->
                        Log.d(TAG, "Mtu changed: $mtu")
                    }
                    .enqueue()

        }

        override fun onDeviceDisconnected() {
            characteristic = null
        }

        override fun isRequiredServiceSupported(gatt: BluetoothGatt): Boolean {
            val service = gatt.getService(IFLYOS_SETUP_SERVICE)
            Log.w(TAG, "Try to get service, isMatchService: ${service != null}")
            if (service != null) {
                characteristic = service.getCharacteristic(IFLYOS_SETUP_REQUEST)
                characteristic?.writeType = BluetoothGattCharacteristic.WRITE_TYPE_NO_RESPONSE
                Log.w(TAG, "Try to get service, isMatchCharacteristic: ${characteristic != null}")

                setNotificationCallback(characteristic)
                        .with { _, data ->
                            if (data.value?.isNotEmpty() == true)
                                Log.d(TAG, "NotificationCallback, data: ${String(data.value!!)}")
                            else
                                Log.d(TAG, "NotificationCallback, data: null")
                        }
                enableNotifications(characteristic)
                        .with { _, data ->
                            if (data.value?.isNotEmpty() == true)
                                Log.d(TAG, "enableNotifications, data: ${String(data.value!!)}")
                            else
                                Log.d(TAG, "enableNotifications, data: null")
                        }
                        .fail { _, status ->
                            Log.w(TAG, "enableNotifications failed: ${GattError.parse(status)}")
                        }
                        .enqueue()
                return characteristic != null
            }
            return false
        }
    }

    init {
        this.listener = listener
        val thread = HandlerThread("")
        thread.start()
        val handler = Handler(thread.looper)
        handler.post(keepAliveRunnable)
        this.handler = handler

        setGattCallbacks(object : BleManagerCallbacks {
            override fun onDeviceDisconnecting(device: BluetoothDevice) {
                Log.d(TAG, "onDeviceDisconnecting")
                listener?.onDeviceDisconnecting(device)
            }

            override fun onDeviceDisconnected(device: BluetoothDevice) {
                Log.d(TAG, "onDeviceDisconnected")
                listener?.onDeviceDisconnected(device)
            }

            override fun onDeviceConnected(device: BluetoothDevice) {
                Log.d(TAG, "onDeviceConnected")
                listener?.onDeviceConnected(device)
            }

            override fun onDeviceNotSupported(device: BluetoothDevice) {
                Log.d(TAG, "onDeviceNotSupported")
                listener?.onDeviceNotSupported(device)
            }

            override fun onBondingFailed(device: BluetoothDevice) {
                Log.d(TAG, "onBondingFailed")
                listener?.onBondingFailed(device)
            }

            override fun onServicesDiscovered(device: BluetoothDevice, optionalServicesFound: Boolean) {
                Log.d(TAG, "onServicesDiscovered, optionalServicesFound: $optionalServicesFound")
                listener?.onServicesDiscovered(device, optionalServicesFound)
            }

            override fun onBondingRequired(device: BluetoothDevice) {
                Log.d(TAG, "onBondingRequired")
                listener?.onBondingRequired(device)
            }

            override fun onLinkLossOccurred(device: BluetoothDevice) {
                Log.d(TAG, "onLinkLossOccurred")
                listener?.onLinkLossOccurred(device)
            }

            override fun onBonded(device: BluetoothDevice) {
                Log.d(TAG, "onBonded")
                listener?.onBonded(device)
            }

            override fun onDeviceReady(device: BluetoothDevice) {
                Log.d(TAG, "onDeviceReady")
                listener?.onDeviceReady(device)
            }

            override fun onError(device: BluetoothDevice, message: String, errorCode: Int) {
                Log.d(TAG, "onError, message: $message, errorCode: $errorCode")
                listener?.onError(device, message, errorCode)
            }

            override fun onDeviceConnecting(device: BluetoothDevice) {
                Log.d(TAG, "onDeviceConnecting")
                listener?.onDeviceConnecting(device)
            }

        })

        Log.d(TAG, "Try to connect device{name=${device.name} address=${device.address}}")
        connect(device)
                .retry(10, 200)
                .done {
                    Log.d(TAG, "connect success")
                }
                .fail { device, status ->
                    Log.d(TAG, "BLE connect failed, status: $status, ${GattError.parse(status)}")
                    listener?.onDeviceConnectFailed(device, status)
                }.enqueue()
    }

    override fun getGattCallback(): BleManagerGattCallback {
        return gattCallback
    }

    fun readDeviceIdSync(
            with: DataReceivedCallback = DataReceivedCallback { _, _ -> },
            fail: FailCallback = FailCallback { _, _ -> }
    ) {
        if (characteristic != null)
            readCharacteristic(characteristic)
                    .with(with)
                    .fail(fail)
                    .enqueue()
        else
            fail.onRequestFailed(
                    device,
                    FailCallback.REASON_DEVICE_NOT_SUPPORTED
            )
    }

    fun sendDisconnect(
            with: DataSentCallback = DataSentCallback { _, _ -> },
            fail: FailCallback = FailCallback { _, _ -> }) {
        if (characteristic != null) {
            val value = "disconnect"
            writeCharacteristic(characteristic, Data.from(value))
                    .with(with)
                    .fail(fail)
                    .enqueue()
        } else
            fail.onRequestFailed(
                    device,
                    FailCallback.REASON_DEVICE_NOT_SUPPORTED
            )
    }

    fun setupData(
            ssid: String,
            password: String?,
            code: String,
            with: DataSentCallback = DataSentCallback { _, _ -> },
            fail: FailCallback = FailCallback { _, _ -> }
    ) {
        if (characteristic != null) {
            val value = "id $ssid\npwd ${if (password.isNullOrEmpty()) "" else password}\ncode $code"
            writeCharacteristic(characteristic, Data.from(value))
                    .with(with)
                    .fail(fail)
                    .enqueue()
        } else
            fail.onRequestFailed(
                    device,
                    FailCallback.REASON_DEVICE_NOT_SUPPORTED
            )
    }

    fun refreshBleDeviceCache(): Request {
        return super.refreshDeviceCache()
    }

    fun destroy() {
        isDestroyed = true
        listener = null
        characteristic = null
        handler?.removeCallbacks(keepAliveRunnable)
    }

    open class Callback : BleManagerCallbacks {

        override fun onDeviceDisconnecting(device: BluetoothDevice) {
        }

        override fun onDeviceDisconnected(device: BluetoothDevice) {
        }

        override fun onDeviceConnected(device: BluetoothDevice) {
        }

        override fun onDeviceNotSupported(device: BluetoothDevice) {
        }

        override fun onBondingFailed(device: BluetoothDevice) {
        }

        override fun onServicesDiscovered(device: BluetoothDevice, optionalServicesFound: Boolean) {
        }

        override fun onBondingRequired(device: BluetoothDevice) {
        }

        override fun onLinkLossOccurred(device: BluetoothDevice) {
        }

        override fun onBonded(device: BluetoothDevice) {
        }

        override fun onDeviceReady(device: BluetoothDevice) {
        }

        override fun onError(device: BluetoothDevice, message: String, errorCode: Int) {
        }

        override fun onDeviceConnecting(device: BluetoothDevice) {
        }

        open fun onDeviceConnectFailed(device: BluetoothDevice, status: Int) {
        }
    }
}

// Possibly error status
// from https://android.googlesource.com/platform/external/bluetooth/bluedroid/+/android-5.1.0_r1/stack/include/gatt_api.h
//
// * GATT_ILLEGAL_PARAMETER     0x0087 (135)
// * GATT_NO_RESOURCES          0x0080 (128)
// * GATT_INTERNAL_ERROR        0x0081 (129)
// * GATT_WRONG_STATE           0x0082 (130)
// * GATT_DB_FULL               0x0083 (131)
// * GATT_BUSY                  0x0084 (132)
// * GATT_ERROR                 0x0085 (133)
// * GATT_CMD_STARTED           0x0086 (134)
// * GATT_PENDING               0x0088 (136)
// * GATT_AUTH_FAIL             0x0089 (137)
// * GATT_MORE                  0x008a (138)
// * GATT_INVALID_CFG           0x008b (139)
// * GATT_SERVICE_STARTED       0x008c (140)
// * GATT_ENCRYPED_MITM         GATT_SUCCESS
// * GATT_ENCRYPED_NO_MITM      0x008d (141)
// * GATT_NOT_ENCRYPTED         0x008e (142)