package com.iflytek.cyber.home.sdk.demo

import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.os.Build
import android.os.Bundle
import android.os.IBinder
import android.util.Log
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.RecyclerView
import com.iflytek.home.sdk.IFlyHome
import com.iflytek.home.sdk.push.OsPushService
import kotlinx.android.synthetic.main.activity_push_demo.*
import java.text.SimpleDateFormat
import java.util.*

class PushDemoActivity : AppCompatActivity() {
    private var service: PushServiceDemo? = null

    private val deviceIdsAdapter = ListAdapter()
    private val userIdsAdapter = ListAdapter()

    private val messageListAdapter = MessageListAdapter()

    private val serviceConnection = object : ServiceConnection {
        override fun onServiceDisconnected(name: ComponentName?) {
            service?.let { service ->
                service.removeSubscribeCallback(subscribeCallback)
            }
            service = null
        }

        override fun onServiceConnected(name: ComponentName?, service: IBinder?) {
            if (service is PushServiceDemo.ServiceBinder) {
                val pushService = service.getService()
                pushService.addSubscribeCallback(subscribeCallback)
                this@PushDemoActivity.service = pushService

                initDeviceIdsUI()

                initUserIdsUI()
            }
        }
    }
    private val subscribeCallback = object : OsPushService.SubscribeCallback() {
        override fun onDeviceMediaStateSubscribed(deviceId: String) {
            super.onDeviceMediaStateSubscribed(deviceId)

            runOnUiThread {
                initDeviceIdsUI()
            }
        }

        override fun onDeviceMediaStateUnsubscribed(deviceId: String) {
            super.onDeviceMediaStateUnsubscribed(deviceId)

            runOnUiThread {
                initDeviceIdsUI()
            }
        }

        override fun onUserDeviceStateSubscribed(userId: String) {
            super.onUserDeviceStateSubscribed(userId)

            runOnUiThread {
                initUserIdsUI()
            }
        }

        override fun onUserDeviceStateUnsubscribed(userId: String) {
            super.onUserDeviceStateSubscribed(userId)

            runOnUiThread {
                initUserIdsUI()
            }
        }

        override fun onMediaStateMessage(deviceId: String, message: String) {
            super.onMediaStateMessage(deviceId, message)

            val textModel = TextModel(
                deviceId, "媒体播放状态更新", message,
                SimpleDateFormat("HH:mm:ss", Locale.getDefault()).format(Date())
            )
            messageListAdapter.list.add(textModel)
            runOnUiThread {
                messageListAdapter.notifyDataSetChanged()

                subscribed_message_list.post {
                    if (messageListAdapter.itemCount > 0)
                        subscribed_message_list.smoothScrollToPosition(messageListAdapter.itemCount - 1)
                }
            }
        }

        override fun onDeviceStateMessage(userId: String, message: String) {
            super.onDeviceStateMessage(userId, message)

            val textModel = TextModel(
                userId, "设备状态更新", message,
                SimpleDateFormat("HH:mm:ss", Locale.getDefault()).format(Date())
            )
            messageListAdapter.list.add(textModel)
            runOnUiThread {
                messageListAdapter.notifyDataSetChanged()

                subscribed_message_list.post {
                    if (messageListAdapter.itemCount > 0)
                        subscribed_message_list.smoothScrollToPosition(messageListAdapter.itemCount - 1)
                }
            }
        }
    }
    private val onSubscribeMessageClickListener = object : OnItemClickListener {
        override fun onItemClick(parent: ViewGroup, itemView: View, position: Int) {
            val textModel = messageListAdapter.list[position]
            val intent = Intent(baseContext, ResultActivity::class.java)
            intent.putExtra("result", textModel.message)
            startActivity(intent)
        }
    }
    private val onUserUnsubscribeListener = object : OnItemClickListener {
        override fun onItemClick(parent: ViewGroup, itemView: View, position: Int) {
            val userId = userIdsAdapter.list[position]
            service?.unsubscribeUserDeviceState(userId)
        }
    }
    private val onDeviceUnsubscribeListener = object : OnItemClickListener {
        override fun onItemClick(parent: ViewGroup, itemView: View, position: Int) {
            val deviceId = deviceIdsAdapter.list[position]
            service?.unsubscribeDeviceMediaState(deviceId)
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (Build.VERSION.SDK_INT >= 23)
            window.decorView.systemUiVisibility =
                window.decorView.systemUiVisibility or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
        setContentView(R.layout.activity_push_demo)
        supportActionBar?.run {
            setHomeButtonEnabled(true)
            setDisplayHomeAsUpEnabled(true)
        }

        val intent = Intent(this, PushServiceDemo::class.java)
        bindService(intent, serviceConnection, Context.BIND_AUTO_CREATE)

        startService(intent) // 除了 bind 还要 startService，否则 unbind 的时候服务就被销毁了

        device_id_list.adapter = deviceIdsAdapter
        user_id_list.adapter = userIdsAdapter

        subscribed_message_list.adapter = messageListAdapter

        subscribe_device_media_state.setOnClickListener {
            val deviceId = et_device_id.text?.toString()
            if (deviceId.isNullOrEmpty()) {
                Toast.makeText(this, "${getString(R.string.device_id)} 不能为空", Toast.LENGTH_SHORT)
                    .show()
            } else {
                val result = service?.subscribeDeviceMediaState(deviceId)
                Log.v("订阅", "尝试订阅 $deviceId 的媒体播放状态, result: $result")
                if (result == IFlyHome.RESULT_ILLEGAL_PARAMS) {
                    Toast.makeText(this, "不能重复订阅设备", Toast.LENGTH_SHORT).show()
                }
            }
        }
        subscribe_user_device_state.setOnClickListener {
            val userId = et_user_id.text?.toString()
            if (userId.isNullOrEmpty()) {
                Toast.makeText(this, "${getString(R.string.user_id)} 不能为空", Toast.LENGTH_SHORT)
                    .show()
            } else {
                val result = service?.subscribeUserDeviceState(userId)
                Log.v("订阅", "尝试订阅 $userId 的设备在离线状态, result: $result")
                if (result == IFlyHome.RESULT_ILLEGAL_PARAMS) {
                    Toast.makeText(this, "不能重复订阅用户", Toast.LENGTH_SHORT).show()
                }
            }
        }
        clear.setOnClickListener {
            messageListAdapter.list.clear()
            messageListAdapter.notifyDataSetChanged()
        }

        messageListAdapter.listener = onSubscribeMessageClickListener
        deviceIdsAdapter.listener = onDeviceUnsubscribeListener
        userIdsAdapter.listener = onUserUnsubscribeListener
    }

    private fun initDeviceIdsUI() {
        service?.let { service ->
            val deviceIds = service.getSubscribedMediaStateDevices()

            deviceIdsAdapter.list.clear()
            deviceIdsAdapter.list.addAll(deviceIds)

            deviceIdsAdapter.notifyDataSetChanged()
        }
    }

    private fun initUserIdsUI() {
        service?.let { service ->
            val userIds = service.getSubscribedDeviceStateUsers()

            userIdsAdapter.list.clear()
            userIdsAdapter.list.addAll(userIds)

            userIdsAdapter.notifyDataSetChanged()
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        unbindService(serviceConnection)
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        if (item?.itemId == android.R.id.home) {
            onBackPressed()
        }
        return super.onOptionsItemSelected(item)
    }

    class ListAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
        val list = mutableListOf<String>()

        var listener: OnItemClickListener? = null

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
            val holder = ItemViewHolder(
                LayoutInflater.from(parent.context)
                    .inflate(R.layout.item_subscribed_item, parent, false)
            )
            holder.unsubscribe?.setOnClickListener {
                listener?.onItemClick(parent, it, holder.adapterPosition)
            }
            return holder
        }

        override fun getItemCount(): Int {
            return list.size
        }

        override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
            if (holder is ItemViewHolder) {
                val text = list[position]

                holder.textView?.text = text
            }
        }
    }

    class MessageListAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
        val list = mutableListOf<TextModel>()
        var listener: OnItemClickListener? = null
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
            val holder = TextViewHolder(
                LayoutInflater.from(parent.context)
                    .inflate(R.layout.item_subscribe_message, parent, false)
            )
            holder.itemView.setOnClickListener {
                listener?.onItemClick(parent, it, holder.adapterPosition)
            }
            return holder
        }

        override fun getItemCount(): Int {
            return list.size
        }

        override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
            if (holder is TextViewHolder) {
                val model = list[position]

                holder.titleView?.text = model.title
                holder.messageView?.text = model.message
                holder.itemIdView?.text = model.id
                holder.timeView?.text = model.time
            }
        }

    }

    class TextViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val titleView: TextView? = itemView.findViewById(R.id.title)
        val itemIdView: TextView? = itemView.findViewById(R.id.item_id)
        val messageView: TextView? = itemView.findViewById(R.id.message)
        val timeView: TextView? = itemView.findViewById(R.id.time)
    }

    class ItemViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val textView: TextView? = itemView.findViewById(R.id.text)
        val unsubscribe: Button? = itemView.findViewById(R.id.unsubscribe)
    }

    interface OnItemClickListener {
        fun onItemClick(parent: ViewGroup, itemView: View, position: Int)
    }

    data class TextModel(
        val id: String?,
        val title: String?,
        val message: String?,
        val time: String?
    )
}