package com.iflytek.cyber.home.sdk.demo

import android.content.Intent
import android.content.res.ColorStateList
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.webkit.WebView
import android.webkit.WebViewClient
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import com.iflytek.home.sdk.IFlyHome
import com.iflytek.home.sdk.callback.AuthResultCallback
import com.iflytek.home.sdk.callback.IFlyHomeCallback
import kotlinx.android.synthetic.main.activity_web_view.*

class WebViewActivity : AppCompatActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (Build.VERSION.SDK_INT >= 23)
            window.decorView.systemUiVisibility =
                window.decorView.systemUiVisibility or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
        setContentView(R.layout.activity_web_view)

        supportActionBar?.setHomeButtonEnabled(true)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        val webTag = intent.getStringExtra(EXTRA_TAG)
        val deviceId = intent.getStringExtra(EXTRA_DEVICE_ID)

        val webViewTag = IFlyHome.register(web_view, object : IFlyHomeCallback() {
            override fun updateHeaderColor(color: String) {
                val colorInt = Color.parseColor(color)
                supportActionBar?.setBackgroundDrawable(ColorDrawable(colorInt))
                window.statusBarColor = Color.rgb(
                    Math.max((Color.red(colorInt) * 0.8).toInt(), 0),
                    Math.max((Color.green(colorInt) * 0.8).toInt(), 0),
                    Math.max((Color.blue(colorInt) * 0.8).toInt(), 0)
                ) // need 20% black cover
            }

            override fun updateTitle(title: String) {
                supportActionBar?.title = title
            }

            override fun openNewPage(tag: String, params: String?) {
                val intent = Intent(this@WebViewActivity, WebViewActivity::class.java)
                intent.putExtra(EXTRA_TAG, tag)
                startActivity(intent)
            }

            override fun closePage() {
                finish()
            }

            override fun getWebViewClient(): WebViewClient? {
                return object : WebViewClient() {
                    override fun onPageFinished(view: WebView?, url: String?) {
                        super.onPageFinished(view, url)
                        go_back.isEnabled = web_view.canGoBack()
                        go_back.imageTintList = ColorStateList.valueOf(
                            if (web_view.canGoBack()) Color.BLACK else Color.parseColor("#A6A6A6")
                        )
                        go_forward.isEnabled = web_view.canGoForward()
                        go_forward.imageTintList = ColorStateList.valueOf(
                            if (web_view.canGoForward()) Color.BLACK else Color.parseColor("#A6A6A6")
                        )
                    }
                }
            }
        }, webTag)

        if (webTag.isNullOrEmpty()) {
            if (intent.hasExtra(EXTRA_URL)) {
                val url = intent.getStringExtra(EXTRA_URL)
                url?.let {
                    handleResult(IFlyHome.openAuthorizePage(web_view, url))
                }
            } else if (intent.hasExtra(EXTRA_AUTH)) {
                val auth = intent.getStringExtra(EXTRA_AUTH)
                handleResult(IFlyHome.openAuthorizePage(web_view, auth, authResultCallback))
            } else if (intent.hasExtra(EXTRA_AUTH_CODE)) {
                val authCode = intent.getIntExtra(EXTRA_AUTH_CODE, -1)
                if (authCode != -1)
                    handleResult(IFlyHome.openAuthorizePage(web_view, authCode, authResultCallback))
                else {
                    Toast.makeText(this@WebViewActivity, "授权码为空", Toast.LENGTH_SHORT).show()
                    finish()
                }
            } else if (intent.hasExtra(EXTRA_TARGET)) {
                val target = intent.getStringExtra(EXTRA_TARGET)
                if (deviceId.isNullOrEmpty()) {
                    handleResult(IFlyHome.openWebPage(webViewTag, target))
                } else {
                    val params = HashMap<String, String>()
                    params["deviceId"] = deviceId
                    handleResult(IFlyHome.openWebPage(webViewTag, target, params))
                }
            } else if (intent.hasExtra(EXTRA_CUSTOM_URL)) {
                web_view.loadUrl(intent.getStringExtra(EXTRA_CUSTOM_URL))
            } else if (intent.hasExtra(EXTRA_TYPE)) {
                IFlyHome.openWebAccounts(webViewTag, intent.getStringExtra(EXTRA_TYPE))
            } else {
                Toast.makeText(this@WebViewActivity, "无法识别意图", Toast.LENGTH_SHORT).show()
                finish()
            }
        }

        go_back.isEnabled = false
        go_back.imageTintList = ColorStateList.valueOf(Color.parseColor("#A6A6A6"))
        go_forward.isEnabled = false
        go_forward.imageTintList = ColorStateList.valueOf(Color.parseColor("#A6A6A6"))
        go_back.setOnClickListener { web_view.goBack() }
        go_forward.setOnClickListener { web_view.goForward() }
    }

    private val authResultCallback = object : AuthResultCallback {
        override fun onFailed(message: String) {
            Toast.makeText(this@WebViewActivity, message, Toast.LENGTH_SHORT).show()
        }

        override fun onSuccess() {
            Toast.makeText(this@WebViewActivity, "授权成功", Toast.LENGTH_SHORT).show()
        }

    }

    private fun handleResult(result: Int) {
        Log.d("WebView", "result: $result")
        when (result) {
            IFlyHome.RESULT_UNINITIALIZED -> {
                showAlert("SDK 未初始化")
            }
            IFlyHome.RESULT_UN_LOGIN -> {
                showAlert("未登录")
            }
            IFlyHome.RESULT_NO_WEB_VIEW -> {
                showAlert("WebView 未注册成功")
            }
            IFlyHome.RESULT_ILLEGAL_PARAMS -> {
                showAlert("参数不合法")
            }
        }
    }

    private fun showAlert(message: String) {
        AlertDialog.Builder(this)
            .setMessage(message)
            .setPositiveButton(android.R.string.yes) { _, _ ->
                finish()
            }.show()
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_web_view, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.itemId) {
            android.R.id.home -> finish()
            R.id.reload -> web_view.reload()
        }
        return super.onOptionsItemSelected(item)
    }

    companion object {
        const val EXTRA_TAG = "tag"
        const val EXTRA_TARGET = "target"
        const val EXTRA_URL = "url"
        const val EXTRA_CUSTOM_URL = "custom_url"
        const val EXTRA_DEVICE_ID = "device_id"
        const val EXTRA_AUTH_CODE = "auth_code"
        const val EXTRA_AUTH = "auth"
        const val EXTRA_TYPE = "type"
    }
}